from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..models.exercise_type import ExerciseType, ExerciseTypeImage, ExerciseTypeTranslation, ExerciseTypeAlternative, \
                                    ExerciseTypeParents, ExerciseTypeDevice


class ExerciseTypeDeviceInline(admin.StackedInline):
    model = ExerciseTypeDevice
    extra = 0
    fields = ["exercise_type", "exercise_device"]


class ImageInline(admin.StackedInline):
    model = ExerciseTypeImage
    extra = 0
    fields = ["name", "url", "type", "get_image_preview"]
    readonly_fields = ("get_image_preview",)

    def get_image_preview(self, obj):
        image_url = '/media/' + str(obj.url)
        if obj.pk:
            return format_html('<img src="{url}" title="{url}" width="30%" height="30%"/> ' \
                               .format(url=image_url))

    get_image_preview.short_description = _("Image Preview")


class TranslationInline(admin.TabularInline):
    model = ExerciseTypeTranslation
    fields = ('language_code', 'name', 'description')
    extra = 0


class ExerciseTypeAlternativesInline(admin.TabularInline):
    model = ExerciseTypeAlternative
    fk_name = 'exercise_type_child'
    fields = ('exercise_type_parent',)
    extra = 0


class ExerciseTypeParentsInline(admin.TabularInline):
    model = ExerciseTypeParents
    fields = ('exercise_tree',)
    extra = 0


class ExerciseTypeAdmin(admin.ModelAdmin):
    list_display = ('exercise_type_id', 'name_colored', 'active', 'base', 'buddy_warning', 'is_description', 'is_desc_hu', 'alternatives')
    search_fields = ['name', 'exercisetypetranslation__name']
    fields = ('name', 'description', 'unit', 'unit_quantity', 'unit_quantity_unit', 'active', 'base', 'buddy_warning')
    list_filter = ('active','base', 'buddy_warning', 'exercisetypeparents__exercise_tree')
    actions = ('clone',)

    @admin.action(description=_('Clone exercise type'))
    def clone(modeladmin, request, queryset):
        for qs in queryset:
            qs.pk = None
            qs.name = qs.name + "_copy"
            qs.save()

    def name_colored(self, obj):
        if obj.active:
            if obj.base:
                color_code = '7bc863'
            else:
                color_code = '4178bc'
        else:
            color_code = 'C20000'
        html = '<span style="color: #{};">{}</span>˓→'.format(color_code, obj.name)
        return format_html(html)

    name_colored.admin_order_field = 'name'
    name_colored.short_description = 'name'

    def is_description(self, obj):
        if not obj.description:
            return "-"
        else:
            return "OK"

    is_description.short_description = 'desc'

    def is_desc_hu(self, obj):
        desc_hu = ",".join([k.description for k in obj.exercisetypetranslation_set.all()])
        if not desc_hu:
            return "-"
        else:
            return "OK"

    is_desc_hu.short_description = 'desc HU'

    def alternatives(self, obj):
        alternatives = []
        for k in obj.exercise_type_child.all():
            alternatives.append(k.exercise_type_parent_id)
        return ", " . join([ExerciseType.objects.get(pk=id).name for id in alternatives])

    alternatives.short_description = 'alternatives'

    inlines = [
        ImageInline,
        TranslationInline,
        ExerciseTypeDeviceInline,
        ExerciseTypeAlternativesInline,
        ExerciseTypeParentsInline
    ]
    exclude = ('exercise_tree',)


admin.site.register(ExerciseType, ExerciseTypeAdmin)
admin.autodiscover()
