from adminsortable2.admin import SortableAdminMixin, SortableInlineAdminMixin
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from .inline_select_action import BaseInlineSelectActionsMixin
from ..models.training_plan import TrainingPlan, TrainingPlanDetail, TrainingPlanTranslation


class TrainingPlanDetailInline(BaseInlineSelectActionsMixin, SortableInlineAdminMixin, admin.TabularInline):
    model = TrainingPlanDetail
    extra = 0
    list_display = (
        'sort', 'training_plan', 'exercise_type', 'set', 'repeats', 'max', 'weight', 'calc',
        'resting_time', 'parallel', 'day')
    list_editable = (
        'sort', 'exercise_type', 'set', 'repeats', 'weight', 'resting_time', 'parallel', 'day')
    list_display_links = ('sort', )
    ordering = ('sort',)

    inline_select_actions = ['clone',]

    def repeat_max(self, obj):
        if obj.repeat_max:
            obj.repeats = -1
        else:
            obj.repeats = 0

    def get_training_plan_info_info(self):
        first_row = _("Special cases: Weight = -1 IF the Weight should be calculated.")
        second_row = _("Repeats = -1 IF the repeats should be maximized")
        return first_row + second_row

    def changelist_view(self, request, extra_context=None):
        response = super(TrainingPlanDetailAdmin, self).changelist_view(request, extra_context)
        extra_context = {
            'extra_hint': self.get_training_plan_info_info()
        }
        try:
            response.context_data.update(extra_context)
        except Exception as e:
            pass
        return response

    def get_inline_actions(self, request, obj=None):
        actions = super(TrainingPlanDetailInline, self).get_inline_actions(request, obj)
        return actions

    @admin.action(description='clone')
    def clone(self, request, obj, parent_obj=None):

        name = str(request.queryset[0].training_plan)
        details = TrainingPlanDetail.objects.filter(training_plan__name=name).order_by('sort')
        new_sort = details[len(details) - 1].sort + 1

        for objectAttr in request.queryset:
            objectAttr.pk = None
            objectAttr.sort = new_sort
            objectAttr.save()
            new_sort = new_sort + 1

    clone.short_description = _("Clone the selected Training Plan Detail")

    def select_max_repeat(self, request, queryset):
        for obj in queryset:
            obj.repeats = -1
            obj.save()

    select_max_repeat.short_description = _("Select the exercises with MAX repeats")

    def select_weight_calc(self, request, queryset):
        for obj in queryset:
            obj.weight = -1
            obj.save()

    select_weight_calc.short_description = _("Select the exercises CALCULATED weight")

    def max(self, obj):
        color_code = "000000"
        if obj.repeats and obj.repeats == -1:
            color_code = 'C20000'

        html = '<span style="color: #{};font-size:32px">{}</span>'.format(color_code, "*")
        return format_html(html)

    def calc(self, obj):
        color_code = "000000"
        if obj.weight == -1:
            color_code = 'C20000'

        html = '<span style="color: #{};font-size:32px">{}</span>'.format(color_code, "*")
        return format_html(html)


class TranslationTrainingPlanInline(admin.TabularInline):
    model = TrainingPlanTranslation
    fields = ('language_code', 'name_translation', 'description_translation')
    extra = 0


class TrainingPlanAdmin( admin.ModelAdmin):
    list_display = ('training_plan_id', 'name', 'internal_name', 'free', 'active')
    fields = ('tree','training_program', 'name', 'description', 'internal_name', 'free', 'active', 'weeks', 'next_training_plan')
    list_editable = ('name', 'internal_name', 'free', 'active')

    def save_model(self, request, obj, form, change):
        print(request)
        super(TrainingPlanAdmin, self).save_model(request, obj, form, change)

    inlines = [
        TranslationTrainingPlanInline,
        TrainingPlanDetailInline
    ]

    @admin.action(description='clone selected training plans')
    def clone(self, request, queryset):
        for trainingPlan in queryset:
            print("clone " + trainingPlan.name)
            id = trainingPlan.training_plan_id
            trainingPlan.pk = None
            trainingPlan.name = trainingPlan.name + " COPY"
            trainingPlan.internal_name = trainingPlan.internal_name + "_copy"
            trainingPlan.active = False
            trainingPlan.save()
            newId = trainingPlan.pk

            details = TrainingPlanDetail.objects.filter(training_plan__training_plan_id=id)
            for detail in details:
                detail.pk = None
                detail.training_plan_id = newId
                detail.save()

            translations = TrainingPlanTranslation.objects.filter(training_plan__training_plan_id=id)
            for translation in translations:
                translation.pk = None
                translation.training_plan_id = newId
                translation.name_translation = translation.name_translation + " COPY"
                translation.save()

    actions = ['clone']


class TrainingPlanDetailAdmin(SortableAdminMixin, admin.ModelAdmin):
    list_display = (
        'sort', 'training_plan', 'exercise_type', 'set', 'repeats',  'weight', 'max','calc',
        'resting_time', 'parallel', 'day', 'ecto_repeats',  'ecto_weight', 'endo_repeats',  'endo_weight', 'round', 'round_group')
    list_filter = ('training_plan__name', 'exercise_type__name')
    list_editable = (
        'exercise_type', 'set', 'repeats', 'weight', 'resting_time', 'parallel', 'day')
    ordering = ('sort',)

    def repeat_max(self, obj):
        if obj.repeat_max:
            obj.repeats = -1
        else:
            obj.repeats = 0

    def get_training_plan_info_info(self):
        first_row = _("Special cases: Weight = -1 IF the Weight should be calculated.")
        second_row = _("Repeats = -1 IF the repeats should be maximized")
        return first_row + second_row

    def changelist_view(self, request, extra_context=None):
        response = super(TrainingPlanDetailAdmin, self).changelist_view(request, extra_context)
        extra_context = {
            'extra_hint': self.get_training_plan_info_info()
        }
        try:
            response.context_data.update(extra_context)
        except Exception as e:
            pass
        return response

    def copy_attributes(self, request, queryset):
        name = str(queryset[0].training_plan)
        details = TrainingPlanDetail.objects.filter(training_plan__name=name).order_by('sort')
        new_sort = details[len(details) - 1].sort + 1

        for objectAttr in queryset:
            objectAttr.pk = None
            objectAttr.sort = new_sort
            objectAttr.save()
            new_sort = new_sort + 1

    copy_attributes.short_description = _("Clone the selected Training Plan Detail")

    def select_max_repeat(self, request, queryset):
        for obj in queryset:
            obj.repeats = -1
            obj.save()

    select_max_repeat.short_description = _("Select the exercises with MAX repeats")

    def select_weight_calc(self, request, queryset):
        for obj in queryset:
            obj.weight = -1
            obj.save()

    select_weight_calc.short_description = _("Select the exercises CALCULATED weight")

    def max(self, obj):
        color_code = "000000"
        if obj.repeats and obj.repeats == -1:
            color_code = 'C20000'

        html = '<span style="color: #{};font-size:32px">{}</span>'.format(color_code, "*")
        return format_html(html)

    def calc(self, obj):
        color_code = "000000"
        if obj.weight == -1:
            color_code = 'C20000'

        html = '<span style="color: #{};font-size:32px">{}</span>'.format(color_code, "*")
        return format_html(html)

    actions = [copy_attributes, select_max_repeat, select_weight_calc]


admin.site.register(TrainingPlan, TrainingPlanAdmin)
admin.site.register(TrainingPlanDetail, TrainingPlanDetailAdmin)
admin.autodiscover()
